// Mandelbox2D_MaxRadius_v1
// Added a MaxRadius param that determines what region circle inversion is applied in
//     (but separate from circle of inversion, which is still set with InvRadius)
// Includes overlay display for visualization of params
// Initial version by CozyG (aka Gregg Helt)
// 2D framework and escape-time coloring based on approach
//     used in Fragmentarium/Examples/Kali.frag

#define providesInit
#info Mandelbox2D_MaxRadius_v1

#include "Progressive2DColoring.frag"

#group Mandelboxen
uniform float Scale; slider[-5,2,5]
uniform float BoxFoldLimit; slider[-5,1,5]
uniform vec2 BoxCenter; slider[(-5,-5),(0,0),(5,5)]
uniform vec2 InvCenter; slider[(-5,-5),(0,0),(5,5)]
uniform float InvRadius; slider[0.001,1.2,5]
uniform float MinRadius; slider[0.0,0.4,5]
uniform float MaxRadius; slider[0.0,1.0,10]
uniform bool UseMaxRadius; checkbox[false]

#group ParamOverlay
uniform bool ShowInversionOverlay; checkbox[false]
uniform bool ShowBoxfoldOverlay; checkbox[false]
uniform bool ShowTestPoint; checkbox[false]
uniform vec2 TestPoint; slider[(-10,-10),(0.3846,1.5),(10,10)]

// TestPoint Mode == 0, test point is point before boxfold
// TestPoint Mode == 1, test point is point after boxfold (allows more direct testing of inversion effect)
uniform int TestPointMode; slider[0,0,1]    

uniform vec3 InvColor; color[1,1,0]
uniform vec3 MinColor; color[1,0,0]
uniform vec3 MaxColor; color[0.0,1.0,0.0]
uniform vec3 BoxColor; color[0,1,1]
uniform vec3 TestPointColor; color[1,0,1]
uniform vec3 ScaleColor; color[1,0.5,0.5]
uniform vec3 OffsetColor; color[1,0.54902,0.0784314]
                        
uniform float OverlayLineWidth; slider[3,5,30]
uniform float OverlayPointSize; slider[5,10,60]

const float PI =  3.14159265359;
const float PI2 = 6.28318530718;

float overlayLineCoordWidth;
float overlayPointCoordRadius;
float invRadius2;
float minRadius2;
float maxRadius2;

void init() {
  // trying to keep overlay constant pixel size
  overlayLineCoordWidth = OverlayLineWidth * pixelSize.x / zoomscale / 2.0;
  overlayPointCoordRadius = OverlayPointSize * pixelSize.x / zoomscale;

  invRadius2 = InvRadius * InvRadius;
  minRadius2 = MinRadius * MinRadius;
  if (UseMaxRadius) { 
    maxRadius2 = MaxRadius * MaxRadius;
  }
  else {
    maxRadius2 = invRadius2;
  }
}

void boxFold(inout vec2 p) {
  p -= BoxCenter;
  // fold x
  if (p.x > BoxFoldLimit) { p.x = 2.0*BoxFoldLimit - p.x; } 
  else if (p.x < -BoxFoldLimit) { p.x = -2.0*BoxFoldLimit - p.x; }
  // fold y
  if (p.y > BoxFoldLimit) { p.y = 2.0*BoxFoldLimit - p.y; }
  else if (p.y < -BoxFoldLimit) { p.y = -2.0*BoxFoldLimit -p.y; }
  p += BoxCenter;
}

// modified inversion that separates radius of inversion and
// maximum radius at which inversion should be applied
void inversion(inout vec2 p) {
  p -= InvCenter;
  float pointRadius2 = dot(p,p);
  if (pointRadius2 < minRadius2) {
    // scale by constant
    p *= invRadius2/minRadius2;  
  }
  // test against maxRadius2 rather than invRadius
  else if (pointRadius2 < maxRadius2) {
    // but still use invRadius as circle of inversion
    p *= invRadius2/pointRadius2;
  }
  p += InvCenter;
}

// Mandelbox2D iteration function
//   (called iteratively in Progessive2DColoring.frag)
// for every iteration:
//   p' = (Scale(Inversion(Boxfold(p))) + c
vec2 formula(vec2 p, vec2 c) {
  boxFold(p);
  inversion(p);
  p *= Scale;
  p += c;
  return p;
}


bool colorCircle(vec2 p, float radius, vec3 drawColor, inout vec3 color) {
  vec2 pc = p;
  pc -= InvCenter;
  bool hit = false;
  if (abs(length(pc) - radius) <= overlayLineCoordWidth) {
    color = drawColor;
    hit = true;
  }
  return hit;
}

bool colorBox(vec2 p, inout vec3 color) {
  vec2 pc = p;
  pc -= BoxCenter;
  bool hit = false;
  if ((abs(pc.x - BoxFoldLimit) <= overlayLineCoordWidth) ||
      (abs(pc.y - BoxFoldLimit) <= overlayLineCoordWidth) ||
      (abs(pc.x + BoxFoldLimit) <= overlayLineCoordWidth) ||
      (abs(pc.y + BoxFoldLimit) <= overlayLineCoordWidth)) {
    color = BoxColor;
    hit = true;
  }
  return hit;
}

bool colorTestPoint(vec2 p, inout vec3 color) {
  bool hit = false;
  vec2 offset = vec2(TestPoint.x, TestPoint.y);
  vec2 q = vec2(TestPoint.x, TestPoint.y);
  if (TestPointMode == 0) {
    if (distance(q, p) < overlayPointCoordRadius) {
      color = TestPointColor;
      hit = true;
    }
    if (!hit) {
      boxFold(q);
      if (distance(q, p) < overlayPointCoordRadius) {
        color = BoxColor;
        hit = true;
      }
    }
  }
  else {
    if (distance(q, p) < overlayPointCoordRadius) {
      color = BoxColor;
      hit = true;
    }
  }
  if (!hit) {
    inversion(q);
    if (distance(q, p) < overlayPointCoordRadius) {
      color = InvColor;
      hit = true;
    }
  }
  if (!hit) {
    q *= Scale;
    if (distance(q, p) < overlayPointCoordRadius) {
      color = ScaleColor;
      hit = true;
    }
  }
  if (!hit) {
    q += offset;
    if (distance(q, p) < overlayPointCoordRadius) {
      color = OffsetColor;
      hit = true;
    }
  }
  return hit;
}

bool applyColorOverlay(vec2 p, inout vec3 color) {
  bool hit = false;
  if (ShowTestPoint) {
    hit = colorTestPoint(p, color);
  }
  if (!hit && ShowInversionOverlay) {
    hit = colorCircle(p, InvRadius, InvColor, color);
    if (!hit) { hit = colorCircle(p, MinRadius, MinColor, color); }
    if (!hit && UseMaxRadius) { hit = colorCircle(p, MaxRadius, MaxColor, color); }
  }
  if (!hit && ShowBoxfoldOverlay) {
    hit = colorBox(p, color);
  }
  return hit;
}

#preset Default
Gamma = 1.2
Brightness = 1.58537
Contrast = 1
Saturation = 0.972222
Center = 0,0
Zoom = 0.150352
ToneMapping = 1
Exposure = 1
AARange = 2
AAExp = 1
GaussianAA = true
Iterations = 8
PreIterations = 1
R = 0.13235
G = 0.55807
B = 1
C = 0.3
EscapeSize = 1.2
ColoringType = 1
ColorFactor = 0.15
FlattenEscapeColor = true
GrayScale = false
Fade = 1
EscapeColor = 0,0,0
Scale = 2
BoxFoldLimit = 1
BoxCenter = 0,0
InvCenter = 0,0
InvRadius = 1.3
MinRadius = 0.6
MaxRadius = 1
UseMaxRadius = false
ShowInversionOverlay = false
ShowBoxfoldOverlay = false
ShowTestPoint = false
TestPoint = 0.3846,1.5
TestPointMode = 0
InvColor = 1,1,0
MinColor = 1,0,0
MaxColor = 0,1,0
BoxColor = 0,1,1
TestPointColor = 1,0,1
ScaleColor = 1,0.5,0.5
OffsetColor = 1,0.54902,0.0784314
OverlayLineWidth = 5
OverlayPointSize = 10
#endpreset

#preset MaxRadius_equals_InvRadius
Gamma = 1.2
Brightness = 1.58537
Contrast = 1
Saturation = 0.972222
Center = 0,0
Zoom = 0.150352
ToneMapping = 1
Exposure = 1
AARange = 2
AAExp = 1
GaussianAA = true
Iterations = 8
PreIterations = 1
R = 0.13235
G = 0.55807
B = 1
C = 0.3
EscapeSize = 1.2
ColoringType = 1
ColorFactor = 0.15
FlattenEscapeColor = true
GrayScale = false
Fade = 1
EscapeColor = 0,0,0
Scale = 2
BoxFoldLimit = 1
BoxCenter = 0,0
InvRadius = 1.3
MinRadius = 0.6
InvCenter = 0,0
ShowInversionOverlay = true
ShowBoxfoldOverlay = false
ShowTestPoint = false
TestPoint = 0.3846,1.5
TestPointMode = 0
InvColor = 1,1,0
MinColor = 1,0,0
BoxColor = 0,1,1
TestPointColor = 1,0,1
ScaleColor = 1,0.5,0.5
OffsetColor = 1,0.54902,0.0784314
MaxRadius = 1
UseMaxRadius = false
MaxColor = 0,1,0
OverlayLineWidth = 5
OverlayPointSize = 10
#endpreset

#preset MaxRadius_less_than_InvRadius
Gamma = 1.2
Brightness = 1.58537
Contrast = 1
Saturation = 0.972222
Center = 0,0
Zoom = 0.150352
ToneMapping = 1
Exposure = 1
AARange = 2
AAExp = 1
GaussianAA = true
Iterations = 8
PreIterations = 1
R = 0.13235
G = 0.55807
B = 1
C = 0.3
EscapeSize = 1.2
ColoringType = 1
ColorFactor = 0.15
FlattenEscapeColor = true
GrayScale = false
Fade = 1
EscapeColor = 0,0,0
Scale = 2
BoxFoldLimit = 1
BoxCenter = 0,0
InvCenter = 0,0
InvRadius = 1.3
MinRadius = 0.6
MaxRadius = 0.9
UseMaxRadius = true
ShowInversionOverlay = true
ShowBoxfoldOverlay = false
ShowTestPoint = false
TestPoint = 0.3846,1.5
TestPointMode = 0
InvColor = 1,1,0
MinColor = 1,0,0
MaxColor = 0,1,0
BoxColor = 0,1,1
TestPointColor = 1,0,1
ScaleColor = 1,0.5,0.5
OffsetColor = 1,0.54902,0.0784314
OverlayLineWidth = 5
OverlayPointSize = 10
#endpreset

#preset MaxRadius_greater_than_InvRadius
Gamma = 1.2
Brightness = 1.58537
Contrast = 1
Saturation = 0.972222
Center = 0,0
Zoom = 0.150352
ToneMapping = 1
Exposure = 1
AARange = 2
AAExp = 1
GaussianAA = true
Iterations = 8
PreIterations = 1
R = 0.13235
G = 0.55807
B = 1
C = 0.3
EscapeSize = 1.2
ColoringType = 1
ColorFactor = 0.15
FlattenEscapeColor = true
GrayScale = false
Fade = 1
EscapeColor = 0,0,0
Scale = 2
BoxFoldLimit = 1
BoxCenter = 0,0
InvCenter = 0,0
InvRadius = 1.3
MinRadius = 0.6
MaxRadius = 2.5
UseMaxRadius = true
ShowInversionOverlay = true
ShowBoxfoldOverlay = false
ShowTestPoint = false
TestPoint = 0.3846,1.5
TestPointMode = 0
InvColor = 1,1,0
MinColor = 1,0,0
MaxColor = 0,1,0
BoxColor = 0,1,1
TestPointColor = 1,0,1
ScaleColor = 1,0.5,0.5
OffsetColor = 1,0.54902,0.0784314
OverlayLineWidth = 5
OverlayPointSize = 10
#endpreset

